@@ -4,8 +4,13 @@
 import os
 import glob
 import sys
-
+import signal
 import unittest
+import subprocess
+import atexit
+
+import pytest
+
 
 # this was renamed in Python 3, provide backwards compatible name
 if sys.version_info[:2] == (2, 7):
@@ -15,11 +20,35 @@ if sys.version_info[0] == 3:
     unittest.TestCase.assertRegexpMatches = unittest.TestCase.assertRegex
     unittest.TestCase.assertRaisesRegexp = unittest.TestCase.assertRaisesRegex
 
-
 if '--help' in sys.argv:
     print("Usage: ./runtests.py <testfiles>")
     sys.exit(0)
 
+
+def dbus_launch_session():
+    if os.name == "nt" or sys.platform == "darwin":
+        return (-1, "")
+
+    try:
+        out = subprocess.check_output([
+            "dbus-daemon", "--session", "--fork", "--print-address=1",
+            "--print-pid=1"])
+    except (subprocess.CalledProcessError, OSError):
+        return (-1, "")
+    else:
+        if sys.version_info[0] == 3:
+            out = out.decode("utf-8")
+        addr, pid = out.splitlines()
+        return int(pid), addr
+
+
+pid, addr = dbus_launch_session()
+if pid >= 0:
+    os.environ["DBUS_SESSION_BUS_ADDRESS"] = addr
+    atexit.register(os.kill, pid, signal.SIGKILL)
+else:
+    os.environ["DBUS_SESSION_BUS_ADDRESS"] = "."
+
 mydir = os.path.dirname(os.path.abspath(__file__))
 tests_builddir = os.path.abspath(os.environ.get('TESTS_BUILDDIR', os.path.dirname(__file__)))
 builddir = os.path.dirname(tests_builddir)
@@ -95,12 +124,11 @@ else:
     for filename in glob.iglob(os.path.join(mydir, 'test_*.py')):
         names.append(os.path.basename(filename)[:-3])
 
-loader = unittest.TestLoader()
-suite = loader.loadTestsFromNames(names)
+
+def unittest_to_pytest_name(name):
+    parts = name.split(".")
+    parts[0] = os.path.join(mydir, parts[0] + ".py")
+    return "::".join(parts)
 
 
-# Run tests.
-runner = unittest.TextTestRunner(verbosity=2)
-result = runner.run(suite)
-if not result.wasSuccessful():
-    sys.exit(1)  # exit code so "make check" reports error
+sys.exit(pytest.main([unittest_to_pytest_name(n) for n in names]))
