@@ -1,6 +1,8 @@
 # -*- Mode: Python -*-
 # encoding: UTF-8
 
+import os
+import sys
 import unittest
 import os.path
 import warnings
@@ -9,23 +11,23 @@ import subprocess
 from gi.repository import GLib
 from gi import PyGIDeprecationWarning
 
-from compathelper import _unicode, _bytes
-
 
 class TestGLib(unittest.TestCase):
+
+    @unittest.skipIf(os.name == "nt", "no bash on Windows")
     def test_find_program_in_path(self):
         bash_path = GLib.find_program_in_path('bash')
-        self.assertTrue(bash_path.endswith('/bash'))
+        self.assertTrue(bash_path.endswith(os.path.sep + 'bash'))
         self.assertTrue(os.path.exists(bash_path))
 
         self.assertEqual(GLib.find_program_in_path('non existing'), None)
 
     def test_markup_escape_text(self):
-        self.assertEqual(GLib.markup_escape_text(_unicode('a&bä')), 'a&amp;bä')
-        self.assertEqual(GLib.markup_escape_text(_bytes('a&b\x05')), 'a&amp;b&#x5;')
+        self.assertEqual(GLib.markup_escape_text(u'a&bä'), 'a&amp;bä')
+        self.assertEqual(GLib.markup_escape_text(b'a&b\x05'), 'a&amp;b&#x5;')
 
         # with explicit length argument
-        self.assertEqual(GLib.markup_escape_text(_bytes('a\x05\x01\x02'), 2), 'a&#x5;')
+        self.assertEqual(GLib.markup_escape_text(b'a\x05\x01\x02', 2), 'a&#x5;')
 
     def test_progname(self):
         GLib.set_prgname('moo')
@@ -37,9 +39,9 @@ class TestGLib(unittest.TestCase):
 
     def test_xdg_dirs(self):
         d = GLib.get_user_data_dir()
-        self.assertTrue('/' in d, d)
+        self.assertTrue(os.path.sep in d, d)
         d = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DESKTOP)
-        self.assertTrue('/' in d, d)
+        self.assertTrue(os.path.sep in d, d)
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', PyGIDeprecationWarning)
 
@@ -48,7 +50,7 @@ class TestGLib(unittest.TestCase):
                              GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC))
 
         for d in GLib.get_system_config_dirs():
-            self.assertTrue('/' in d, d)
+            self.assertTrue(os.path.sep in d, d)
         for d in GLib.get_system_data_dirs():
             self.assertTrue(isinstance(d, str), d)
 
@@ -60,12 +62,12 @@ class TestGLib(unittest.TestCase):
         self.assertEqual(GLib.filename_display_basename('bar/foo'), 'foo')
 
         # this is locale dependent, so we cannot completely verify the result
-        res = GLib.filename_from_utf8(_unicode('aäb'))
+        res = GLib.filename_from_utf8(u'aäb')
         self.assertTrue(isinstance(res, bytes))
         self.assertGreaterEqual(len(res), 3)
 
         # with explicit length argument
-        self.assertEqual(GLib.filename_from_utf8(_unicode('aäb'), 1), b'a')
+        self.assertEqual(GLib.filename_from_utf8(u'aäb', 1), b'a')
 
     def test_uri_extract(self):
         res = GLib.uri_list_extract_uris('''# some comment
@@ -85,6 +87,7 @@ https://my.org/q?x=1&y=2
         self.assertTrue(isinstance(tm, float))
         self.assertGreater(tm, 1350000000.0)
 
+    @unittest.skipIf(sys.platform == "darwin", "fails on OSX")
     def test_main_loop(self):
         # note we do not test run() here, as we use this in countless other
         # tests
@@ -122,58 +125,74 @@ https://my.org/q?x=1&y=2
         self.assertTrue(context.pending() in [True, False])
         self.assertTrue(context.iteration(False) in [True, False])
 
+    @unittest.skipIf(os.name == "nt", "hangs")
     def test_io_add_watch_no_data(self):
         (r, w) = os.pipe()
         call_data = []
 
         def cb(fd, condition):
             call_data.append((fd, condition, os.read(fd, 1)))
+            if len(call_data) == 2:
+                ml.quit()
             return True
 
         # io_add_watch() takes an IOChannel, calling with an fd is deprecated
         with warnings.catch_warnings(record=True) as warn:
             warnings.simplefilter('always')
-            GLib.io_add_watch(r, GLib.IOCondition.IN, cb)
+            GLib.io_add_watch(r, GLib.IOCondition.IN, cb,
+                              priority=GLib.PRIORITY_HIGH)
             self.assertTrue(issubclass(warn[0].category, PyGIDeprecationWarning))
 
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
         ml = GLib.MainLoop()
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(call_data, [(r, GLib.IOCondition.IN, b'a'),
                                      (r, GLib.IOCondition.IN, b'b')])
 
+    @unittest.skipIf(os.name == "nt", "hangs")
     def test_io_add_watch_with_data(self):
         (r, w) = os.pipe()
         call_data = []
 
         def cb(fd, condition, data):
             call_data.append((fd, condition, os.read(fd, 1), data))
+            if len(call_data) == 2:
+                ml.quit()
             return True
 
         # io_add_watch() takes an IOChannel, calling with an fd is deprecated
         with warnings.catch_warnings(record=True) as warn:
             warnings.simplefilter('always')
-            GLib.io_add_watch(r, GLib.IOCondition.IN, cb, 'moo')
+            GLib.io_add_watch(r, GLib.IOCondition.IN, cb, 'moo',
+                              priority=GLib.PRIORITY_HIGH)
             self.assertTrue(issubclass(warn[0].category, PyGIDeprecationWarning))
 
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
         ml = GLib.MainLoop()
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(call_data, [(r, GLib.IOCondition.IN, b'a', 'moo'),
                                      (r, GLib.IOCondition.IN, b'b', 'moo')])
 
+    @unittest.skipIf(os.name == "nt", "hangs")
     def test_io_add_watch_with_multiple_data(self):
         (r, w) = os.pipe()
         call_data = []
 
         def cb(fd, condition, *user_data):
             call_data.append((fd, condition, os.read(fd, 1), user_data))
+            ml.quit()
             return True
 
         # io_add_watch() takes an IOChannel, calling with an fd is deprecated
@@ -183,17 +202,19 @@ https://my.org/q?x=1&y=2
             self.assertTrue(issubclass(warn[0].category, PyGIDeprecationWarning))
 
         ml = GLib.MainLoop()
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, ml.quit)
+        GLib.idle_add(lambda: os.write(w, b'a') and False)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(call_data, [(r, GLib.IOCondition.IN, b'a', ('moo', 'foo'))])
 
+    @unittest.skipIf(sys.platform == "darwin", "fails")
+    @unittest.skipIf(os.name == "nt", "no shell on Windows")
     def test_io_add_watch_pyfile(self):
         call_data = []
 
-        cmd = subprocess.Popen('sleep 0.1; echo hello; sleep 0.2; echo world',
-                               shell=True, stdout=subprocess.PIPE)
+        cmd = subprocess.Popen('echo hello; echo world',
+                               shell=True, bufsize=0, stdout=subprocess.PIPE)
 
         def cb(file, condition):
             call_data.append((file, condition, file.readline()))
