@@ -1,19 +1,21 @@
 # -*- Mode: Python -*-
 # encoding: UTF-8
-from __future__ import unicode_literals
 
+import os
 import unittest
 import tempfile
 import os.path
-import fcntl
 import shutil
 import warnings
 
+try:
+    import fcntl
+except ImportError:
+    fcntl = None
+
 from gi.repository import GLib
 from gi import PyGIDeprecationWarning
 
-from compathelper import _unicode
-
 
 class IOChannel(unittest.TestCase):
     def setUp(self):
@@ -21,7 +23,7 @@ class IOChannel(unittest.TestCase):
 
         self.testutf8 = os.path.join(self.workdir, 'testutf8.txt')
         with open(self.testutf8, 'wb') as f:
-            f.write('''hello ♥ world
+            f.write(u'''hello ♥ world
 second line
 
 À demain!'''.encode('UTF-8'))
@@ -42,11 +44,11 @@ second line
         ch = GLib.IOChannel(filename=self.testutf8)
         self.assertEqual(ch.get_encoding(), 'UTF-8')
         self.assertTrue(ch.get_close_on_unref())
-        self.assertEqual(_unicode(ch.readline()), 'hello ♥ world\n')
+        self.assertEqual(ch.readline(), 'hello ♥ world\n')
         self.assertEqual(ch.get_buffer_condition(), GLib.IOCondition.IN)
         self.assertEqual(ch.readline(), 'second line\n')
         self.assertEqual(ch.readline(), '\n')
-        self.assertEqual(_unicode(ch.readline()), 'À demain!')
+        self.assertEqual(ch.readline(), 'À demain!')
         self.assertEqual(ch.get_buffer_condition(), 0)
         self.assertEqual(ch.readline(), '')
         ch.shutdown(True)
@@ -55,10 +57,10 @@ second line
         ch = GLib.IOChannel(filename=self.testlatin1, mode='r')
         ch.set_encoding('latin1')
         self.assertEqual(ch.get_encoding(), 'latin1')
-        self.assertEqual(_unicode(ch.readline()), 'hellø world\n')
+        self.assertEqual(ch.readline(), 'hellø world\n')
         self.assertEqual(ch.readline(), 'second line\n')
         self.assertEqual(ch.readline(), '\n')
-        self.assertEqual(_unicode(ch.readline()), 'À demain!')
+        self.assertEqual(ch.readline(), 'À demain!')
         ch.shutdown(True)
 
     def test_file_iter(self):
@@ -67,7 +69,7 @@ second line
         for item in ch:
             items.append(item)
         self.assertEqual(len(items), 4)
-        self.assertEqual(_unicode(items[0]), 'hello ♥ world\n')
+        self.assertEqual(items[0], 'hello ♥ world\n')
         ch.shutdown(True)
 
     def test_file_readlines(self):
@@ -77,8 +79,8 @@ second line
         # empty one
         self.assertGreaterEqual(len(lines), 4)
         self.assertLessEqual(len(lines), 5)
-        self.assertEqual(_unicode(lines[0]), 'hello ♥ world\n')
-        self.assertEqual(_unicode(lines[3]), 'À demain!')
+        self.assertEqual(lines[0], 'hello ♥ world\n')
+        self.assertEqual(lines[3], 'À demain!')
         if len(lines) == 4:
             self.assertEqual(lines[4], '')
 
@@ -112,6 +114,7 @@ second line
 
         # invalid whence value
         self.assertRaises(ValueError, ch.seek, 0, 3)
+        ch.shutdown(True)
 
     def test_file_write(self):
         ch = GLib.IOChannel(filename=self.testout, mode='w')
@@ -124,7 +127,7 @@ second line
         ch.shutdown(True)
 
         with open(self.testout, 'rb') as f:
-            self.assertEqual(f.read().decode('latin1'), 'hellø world\nÀ demain!')
+            self.assertEqual(f.read().decode('latin1'), u'hellø world\nÀ demain!')
 
     def test_file_writelines(self):
         ch = GLib.IOChannel(filename=self.testout, mode='w')
@@ -167,6 +170,7 @@ second line
         self.assertEqual(reader.read(), b'ghi')
         reader.shutdown(True)
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_fd_read(self):
         (r, w) = os.pipe()
 
@@ -186,6 +190,7 @@ second line
 
         ch.shutdown(True)
 
+    @unittest.skipUnless(fcntl, "no fcntl")
     def test_fd_write(self):
         (r, w) = os.pipe()
         fcntl.fcntl(r, fcntl.F_SETFL, fcntl.fcntl(r, fcntl.F_GETFL) | os.O_NONBLOCK)
@@ -203,6 +208,7 @@ second line
         self.assertEqual(os.read(r, 10), b'\x03\x04')
         os.close(r)
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_deprecated_method_add_watch_no_data(self):
         (r, w) = os.pipe()
 
@@ -216,23 +222,28 @@ second line
             self.assertEqual(channel, ch)
             self.assertEqual(condition, GLib.IOCondition.IN)
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         # io_add_watch() method is deprecated, use GLib.io_add_watch
         with warnings.catch_warnings(record=True) as warn:
             warnings.simplefilter('always')
-            ch.add_watch(GLib.IOCondition.IN, cb)
+            ch.add_watch(GLib.IOCondition.IN, cb, priority=GLib.PRIORITY_HIGH)
             self.assertTrue(issubclass(warn[0].category, PyGIDeprecationWarning))
 
-        ml = GLib.MainLoop()
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
 
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+        ml = GLib.MainLoop()
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_deprecated_method_add_watch_data_priority(self):
         (r, w) = os.pipe()
 
@@ -247,6 +258,8 @@ second line
             self.assertEqual(condition, GLib.IOCondition.IN)
             self.assertEqual(data, 'hello')
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         ml = GLib.MainLoop()
@@ -259,13 +272,17 @@ second line
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
 
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_add_watch_no_data(self):
         (r, w) = os.pipe()
 
@@ -279,6 +296,8 @@ second line
             self.assertEqual(channel, ch)
             self.assertEqual(condition, GLib.IOCondition.IN)
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         id = GLib.io_add_watch(ch, GLib.PRIORITY_HIGH, GLib.IOCondition.IN, cb)
@@ -286,13 +305,18 @@ second line
         ml = GLib.MainLoop()
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_add_watch_with_data(self):
         (r, w) = os.pipe()
 
@@ -307,6 +331,8 @@ second line
             self.assertEqual(condition, GLib.IOCondition.IN)
             self.assertEqual(data, 'hello')
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         id = GLib.io_add_watch(ch, GLib.PRIORITY_HIGH, GLib.IOCondition.IN, cb, 'hello')
@@ -314,13 +340,18 @@ second line
         ml = GLib.MainLoop()
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_add_watch_with_multi_data(self):
         (r, w) = os.pipe()
 
@@ -337,6 +368,8 @@ second line
             self.assertEqual(data2, 'b')
             self.assertEqual(data3, 'c')
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         id = GLib.io_add_watch(ch, GLib.PRIORITY_HIGH, GLib.IOCondition.IN, cb,
@@ -345,13 +378,18 @@ second line
         ml = GLib.MainLoop()
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_deprecated_add_watch_no_data(self):
         (r, w) = os.pipe()
 
@@ -365,6 +403,8 @@ second line
             self.assertEqual(channel, ch)
             self.assertEqual(condition, GLib.IOCondition.IN)
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         with warnings.catch_warnings(record=True) as warn:
@@ -375,13 +415,18 @@ second line
         ml = GLib.MainLoop()
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
 
+    @unittest.skipIf(os.name == "nt", "NONBLOCK not implemented on Windows")
     def test_deprecated_add_watch_with_data(self):
         (r, w) = os.pipe()
 
@@ -396,6 +441,8 @@ second line
             self.assertEqual(condition, GLib.IOCondition.IN)
             self.assertEqual(data, 'hello')
             cb_reads.append(channel.read())
+            if len(cb_reads) == 2:
+                ml.quit()
             return True
 
         with warnings.catch_warnings(record=True) as warn:
@@ -407,9 +454,14 @@ second line
         ml = GLib.MainLoop()
         self.assertEqual(ml.get_context().find_source_by_id(id).priority,
                          GLib.PRIORITY_HIGH)
-        GLib.timeout_add(10, lambda: os.write(w, b'a') and False)
-        GLib.timeout_add(100, lambda: os.write(w, b'b') and False)
-        GLib.timeout_add(200, ml.quit)
+
+        def write():
+            os.write(w, b'a')
+            GLib.idle_add(lambda: os.write(w, b'b') and False)
+
+        GLib.idle_add(write)
+
+        GLib.timeout_add(2000, ml.quit)
         ml.run()
 
         self.assertEqual(cb_reads, [b'a', b'b'])
@@ -423,5 +475,6 @@ second line
             self.assertEqual(GLib.IOFlags.IS_SEEKABLE, GLib.IO_FLAG_IS_SEEKABLE)
             self.assertEqual(GLib.IOStatus.NORMAL, GLib.IO_STATUS_NORMAL)
 
+
 if __name__ == '__main__':
     unittest.main()
